--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Crafting = P:NewModule("Crafting", "AceEvent-3.0")

-- success:  SENT, RECAST DoTradeSkill, SUCCEEDED, PUSH
-- multiple: SENT, RECAST, Do, SUCCEEDED, SENT, SUCCEEDED
-- inventory full: SENT, RECAST, DoTradeSkill, <full>, RECAST*
-- missing mats: DoTradeSkill, FAILED
-- interrupted: SENT, RECAST, Do, INTERRUPTED
-- multi success then interrupted: SENT, RECAST, Do, SUCCESS, SENT, INTERRUPTED

local FAKE_ITEMS = {
	[37168] = true, -- Mysterious Tarot
	[44142] = true, -- Strange Tarot
	[44161] = true, -- arcane tarot
	[44163] = true, -- Shadowy Tarot
	[44316] = true, -- Darkmoon card
	[44317] = true, -- Greater Darkmoon card
	[44318] = true, -- Darkmoon card of the north
	[44508] = true, -- Discovery! (alch research)
	[44680] = true, -- assorted writings (glyph research)
	[49640] = true, -- Essence or dust (Abyss crystal shatter)
	[54464] = true, -- Random Volatile Element (transmute living elements)
}

local function tablecopy(t)
	if type(t)=="table" then
		local rv={}
		for k,v in pairs(t) do
			rv[k]=tablecopy(v)
		end
		setmetatable(rv,getmetatable(t))
		return rv
	else
		return t
	end
end

local function failcb(message)
	local function cb(...)
		args={...}
		P:DebugPrint({message=message,args=args})
	end
	return cb
end

local GEM_CLASS = select(10,GetAuctionItemClasses())

-- Hook crafting stuff
local function Hook_DoTradeSkill(index,count)
	count = count or 1
	local skillName, skillType, numAvailable, isExpanded, serviceType = GetTradeSkillInfo(index)
	local fromItems = P.ItemSet.new()
	local toExpected = nil

	for i=1,GetTradeSkillNumReagents(index) do
		local _, _, reagentCount, _ = GetTradeSkillReagentInfo(index, i)
		local link = GetTradeSkillReagentItemLink(index, i)
		local id = P.idOfItem(link)
		fromItems:Inc(id,reagentCount)
	end
	
	local function onCreate(item,count)
		local toItems = P.ItemSet.new()
		local id = P.idOfItem(item)
		toItems:Inc(id,count)
		CB:RecordTransaction(nil,fromItems,toItems,toExpected)
	end
	
	local function onSpell()
		count = count-1
		if count>0 then
			P:SpellCallback(skillName,onSpell,failcb("crafting spell failed"))
		end
		P:CreateCallback(onCreate,failcb("crafting create failed"))
	end
	
	if serviceType then
		-- modifies something
			local function onItem(item)
				local id = P.idOfItem(item)
				if id~=38682 then return end -- we only care (i think?) if we're targeting Enchanting Vellum (38682)
				fromItems:Inc(id,1)
				P:SpellCallback(skillName,onSpell,failcb("crafting spell failed"))
			end
			P:ItemCallback(onItem,failcb("crafting item failed"))
	else
		-- creates an itme
		local link = GetTradeSkillItemLink(index)
		if link then
			local id=P.idOfItem(link)
			local _,_,quality,_,_,class = GetItemInfo(id)
			-- TODO: for now, we just ignore uncommon gems since we may make perfect ones.  Perfect seems to be 20% chance looking at wowhead comments.  We need a table mapping uncommon gems to expected
			if id and quality and (quality~=2 or class~=GEM_CLASS) then -- GetItemInfo returns nil when the item isn't a real item!  
				local minMade, maxMade = GetTradeSkillNumMade(index)
				toExpected = {[id]=(minMade+maxMade)/2}
			end
		end
		P:SpellCallback(skillName,onSpell,failcb("crafting spell failed"))
	end		
end
hooksecurefunc("DoTradeSkill",Hook_DoTradeSkill)
